# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
import logging
from typing import Any
from unittest import mock

import pytest

from airflow.models import Connection

# Import Operator
from airflow.providers.apache.kafka.operators.consume import VALID_COMMIT_CADENCE, ConsumeFromTopicOperator
from airflow.providers.common.compat.sdk import AirflowException

log = logging.getLogger(__name__)


def _no_op(*args, **kwargs) -> Any:
    """no_op A function that returns its arguments

    :return: whatever was passed in
    :rtype: Any
    """
    return args, kwargs


def create_mock_kafka_consumer(
    message_count: int = 1001, message_content: Any = "test_message", track_consumed_messages: bool = False
) -> tuple[mock.MagicMock, mock.MagicMock, list[int] | None]:
    """
    Creates a mock Kafka consumer with configurable behavior.

    :param message_count: Number of messages to generate
    :param message_content: Content of each message
    :param track_consumed_messages: Whether to track total consumed messages
    :return: Tuple of (mock_consumer, mock_get_consumer, total_consumed_messages)
    """
    # Initialize messages and tracking variables
    mocked_messages = [message_content for _ in range(message_count)]
    total_consumed_count = [0] if track_consumed_messages else None

    # Define the mock consume behavior
    def mock_consume(num_messages=0, timeout=-1):
        nonlocal mocked_messages
        if num_messages < 0:
            raise Exception("Number of messages needs to be positive")

        msg_count = min(num_messages, len(mocked_messages))
        returned_messages = mocked_messages[:msg_count]
        mocked_messages = mocked_messages[msg_count:]

        if track_consumed_messages:
            total_consumed_count[0] += msg_count

        return returned_messages

    # Create mock objects
    mock_consumer = mock.MagicMock()
    mock_consumer.consume = mock_consume

    mock_get_consumer = mock.patch(
        "airflow.providers.apache.kafka.hooks.consume.KafkaConsumerHook.get_consumer",
        return_value=mock_consumer,
    )

    #
    return mock_consumer, mock_get_consumer, total_consumed_count  # type: ignore[return-value]


class TestConsumeFromTopic:
    """
    Test ConsumeFromTopic
    """

    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(
                    {"socket.timeout.ms": 10, "bootstrap.servers": "localhost:9092", "group.id": "test_group"}
                ),
            )
        )

    def test_operator(self):
        operator = ConsumeFromTopicOperator(
            kafka_config_id="kafka_d",
            topics=["test"],
            apply_function="unit.apache.kafka.operators.test_consume._no_op",
            task_id="test",
            poll_timeout=0.0001,
        )

        # execute the operator (this is essentially a no op as the broker isn't setup)
        operator.execute(context={})

    def test_operator_callable(self):
        operator = ConsumeFromTopicOperator(
            kafka_config_id="kafka_d",
            topics=["test"],
            apply_function=_no_op,
            task_id="test",
            poll_timeout=0.0001,
        )

        # execute the operator (this is essentially a no op as the broker isn't setup)
        operator.execute(context={})

    @pytest.mark.parametrize(
        ("max_messages", "expected_consumed_messages"),
        [
            [None, 1001],  # Consume all messages
            [100, 1000],  # max_messages < max_batch_size -> max_messages is set to default max_batch_size
            [2000, 1001],  # max_messages > max_batch_size
        ],
    )
    def test_operator_consume(self, max_messages, expected_consumed_messages):
        # Create mock consumer with tracking of consumed messages
        _, mock_get_consumer, consumed_messages = create_mock_kafka_consumer(
            message_count=1001, message_content="test_messages", track_consumed_messages=True
        )

        # Use the mock
        with mock_get_consumer:
            operator = ConsumeFromTopicOperator(
                kafka_config_id="kafka_d",
                topics=["test"],
                task_id="test",
                poll_timeout=0.0001,
                max_messages=max_messages,
            )

            # execute the operator (this is essentially a no op as we're mocking the consumer)
            operator.execute(context={})
            assert consumed_messages[0] == expected_consumed_messages

    @pytest.mark.parametrize(
        "commit_cadence",
        [
            # will raise AirflowException for invalid commit_cadence
            ("invalid_cadence"),
            ("end_of_operator"),
            ("end_of_batch"),
            ("never"),
        ],
    )
    def test__validate_commit_cadence_on_construct(self, commit_cadence):
        operator_kwargs = {
            "kafka_config_id": "kafka_d",
            "topics": ["test"],
            "task_id": "test",
            "commit_cadence": commit_cadence,
        }
        # early return for invalid commit_cadence
        if commit_cadence == "invalid_cadence":
            with pytest.raises(
                AirflowException,
                match=f"commit_cadence must be one of {VALID_COMMIT_CADENCE}. Got invalid_cadence",
            ):
                ConsumeFromTopicOperator(**operator_kwargs)
            return

        # should not raise AirflowException for valid commit_cadence
        ConsumeFromTopicOperator(**operator_kwargs)

    @pytest.mark.parametrize(
        ("commit_cadence", "enable_auto_commit", "expected_warning"),
        [
            # will not log warning if set 'enable.auto.commit' to false
            ("end_of_operator", "false", False),
            ("end_of_batch", "false", False),
            ("never", "false", False),
            # will log warning if set 'enable.auto.commit' to true
            ("end_of_operator", "true", True),
            ("end_of_batch", "true", True),
            ("never", "true", True),
            # will log warning if 'enable.auto.commit' is not set
            ("end_of_operator", None, True),
            ("end_of_batch", None, True),
            ("never", None, True),
            # will not log warning if commit_cadence is None, no matter the value of 'enable.auto.commit'
            (None, None, False),
            (None, "true", False),
            (None, "false", False),
        ],
    )
    def test__validate_commit_cadence_before_execute(
        self, commit_cadence, enable_auto_commit, expected_warning
    ):
        # mock connection and hook
        mocked_hook = mock.MagicMock()
        mocked_hook.get_connection.return_value.extra_dejson = (
            {} if enable_auto_commit is None else {"enable.auto.commit": enable_auto_commit}
        )

        with (
            mock.patch(
                "airflow.providers.apache.kafka.operators.consume.ConsumeFromTopicOperator.hook",
                new_callable=mock.PropertyMock,
                return_value=mocked_hook,
            ),
            mock.patch(
                "airflow.providers.apache.kafka.operators.consume.ConsumeFromTopicOperator.log"
            ) as mock_log,
        ):
            operator = ConsumeFromTopicOperator(
                kafka_config_id="kafka_d",
                topics=["test"],
                task_id="test",
                commit_cadence=commit_cadence,
            )
            operator._validate_commit_cadence_before_execute()
            if expected_warning:
                expected_warning_template = (
                    "To respect commit_cadence='%s', "
                    "'enable.auto.commit' should be set to 'false' in the Kafka connection configuration. "
                    "Currently, 'enable.auto.commit' is not explicitly set, so it defaults to 'true', which causes "
                    "the consumer to auto-commit offsets every 5 seconds. "
                    "See: https://kafka.apache.org/documentation/#consumerconfigs_enable.auto.commit for more information"
                )
                mock_log.warning.assert_called_with(expected_warning_template, commit_cadence)
            else:
                mock_log.warning.assert_not_called()

    @pytest.mark.parametrize(
        ("commit_cadence", "max_messages", "expected_commit_calls"),
        [
            # end_of_operator: should call commit once at the end
            ("end_of_operator", 1500, 1),
            # end_of_batch: should call commit after each batch (2 batches for 1500 messages with default batch size 1000)
            # and a final commit at the end of execute (since commit_cadence is not 'never')
            ("end_of_batch", 1500, 3),
            # never: should never call commit
            ("never", 1500, 0),
        ],
    )
    def test_commit_cadence_behavior(self, commit_cadence, max_messages, expected_commit_calls):
        # Create mock consumer with 1500 messages (will use 1001 for the first batch)
        mock_consumer, mock_get_consumer, _ = create_mock_kafka_consumer(
            message_count=1001,  # Only need to create 1001 messages for the first batch
        )

        # Use the mocks
        with mock_get_consumer:
            # Create and execute the operator
            operator = ConsumeFromTopicOperator(
                kafka_config_id="kafka_d",
                topics=["test"],
                task_id="test",
                poll_timeout=0.0001,
                max_messages=max_messages,
                commit_cadence=commit_cadence,
                apply_function=_no_op,
            )

            operator.execute(context={})

            # Verify commit was called the expected number of times
            assert mock_consumer.commit.call_count == expected_commit_calls

            # Verify consumer was closed
            mock_consumer.close.assert_called_once()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import asyncio
from collections.abc import Sequence
from functools import partial
from typing import Any

from asgiref.sync import sync_to_async

from airflow.providers.apache.kafka.hooks.consume import KafkaConsumerHook
from airflow.providers.apache.kafka.version_compat import AIRFLOW_V_3_0_PLUS
from airflow.providers.common.compat.module_loading import import_string
from airflow.providers.common.compat.sdk import AirflowException
from airflow.triggers.base import TriggerEvent

if AIRFLOW_V_3_0_PLUS:
    from airflow.triggers.base import BaseEventTrigger
else:
    from airflow.triggers.base import BaseTrigger as BaseEventTrigger  # type: ignore


class AwaitMessageTrigger(BaseEventTrigger):
    """
    A trigger that waits for a message matching specific criteria to arrive in Kafka.

    The behavior of the consumer of this trigger is as follows:
    - poll the Kafka topics for a message, if no message returned, sleep
    - process the message with provided callable and commit the message offset:

        - if callable is provided and returns any data, raise a TriggerEvent with the return data
        - else raise a TriggerEvent with the original message

    - by default, the message offset is committed after processing. This can be
      disabled by setting ``commit_offset=False``, allowing manual offset management
      in downstream tasks.

    :param kafka_config_id: The connection object to use, defaults to "kafka_default"
    :param topics: The topic (or topic regex) that should be searched for messages
    :param apply_function: the location of the function to apply to messages for determination of matching
        criteria. (In python dot notation as a string)
    :param apply_function_args: A set of arguments to apply to the callable, defaults to None
    :param apply_function_kwargs: A set of key word arguments to apply to the callable, defaults to None,
        defaults to None
    :param poll_timeout: How long the Kafka client should wait before returning from a poll request to
        Kafka (seconds), defaults to 1
    :param poll_interval: How long the trigger should sleep after reaching the end of the Kafka log
        (seconds), defaults to 5
    :param commit_offset: Whether to commit the message offset after poll.
        If set to False, the offset is not committed automatically, allowing
        downstream tasks to handle offset committing manually (e.g., after
        successful processing). Defaults to True.

    """

    def __init__(
        self,
        topics: Sequence[str],
        apply_function: str | None = None,
        kafka_config_id: str = "kafka_default",
        apply_function_args: Sequence[Any] | None = None,
        apply_function_kwargs: dict[Any, Any] | None = None,
        poll_timeout: float = 1,
        poll_interval: float = 5,
        commit_offset: bool = True,
    ) -> None:
        self.topics = topics
        self.apply_function = apply_function
        self.apply_function_args = apply_function_args or ()
        self.apply_function_kwargs = apply_function_kwargs or {}
        self.kafka_config_id = kafka_config_id
        self.poll_timeout = poll_timeout
        self.poll_interval = poll_interval
        self.commit_offset = commit_offset

    def serialize(self) -> tuple[str, dict[str, Any]]:
        return (
            "airflow.providers.apache.kafka.triggers.await_message.AwaitMessageTrigger",
            {
                "topics": self.topics,
                "apply_function": self.apply_function,
                "apply_function_args": self.apply_function_args,
                "apply_function_kwargs": self.apply_function_kwargs,
                "kafka_config_id": self.kafka_config_id,
                "poll_timeout": self.poll_timeout,
                "poll_interval": self.poll_interval,
                "commit_offset": self.commit_offset,
            },
        )

    async def run(self):
        consumer_hook = KafkaConsumerHook(topics=self.topics, kafka_config_id=self.kafka_config_id)

        async_get_consumer = sync_to_async(consumer_hook.get_consumer)
        consumer = await async_get_consumer()

        async_poll = sync_to_async(consumer.poll)
        async_commit = sync_to_async(consumer.commit)

        async_message_process = None
        if self.apply_function:
            processing_call = import_string(self.apply_function)
            processing_call = partial(
                processing_call, *self.apply_function_args, **self.apply_function_kwargs
            )
            async_message_process = sync_to_async(processing_call)
        while True:
            message = await async_poll(self.poll_timeout)

            if message is None:
                continue
            elif message.error():
                raise AirflowException(f"Error: {message.error()}")
            else:
                event = (
                    await async_message_process(message)
                    if async_message_process
                    else message.value().decode("utf-8")
                )
                if event:
                    if self.commit_offset:
                        await async_commit(message=message, asynchronous=False)
                    yield TriggerEvent(event)
                    break
                else:
                    if self.commit_offset:
                        await async_commit(message=message, asynchronous=False)
                    await asyncio.sleep(self.poll_interval)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.


from __future__ import annotations

import logging
from typing import Any

import pytest
from confluent_kafka import Producer

# Import Operator
from airflow.models.connection import Connection
from airflow.providers.apache.kafka.operators.consume import ConsumeFromTopicOperator

log = logging.getLogger(__name__)


def _batch_tester(messages, test_string=None):
    assert test_string
    assert len(messages) == 10

    for x in messages:
        # Confluent Kafka converts messages to bytes
        assert x.value().decode(encoding="utf-8") == test_string


def _basic_message_tester(message, test=None) -> Any:
    """a function that tests the message received"""

    assert test
    # Confluent Kafka converts messages to bytes
    assert message.value().decode(encoding="utf-8") == test


@pytest.fixture(autouse=True)
def kafka_consumer_connections(create_connection_without_db):
    """Create Kafka consumer connections for testing purpose."""
    connections = [
        Connection(
            conn_id="operator.consumer.test.integration.test_1",
            uri="kafka://broker:29092?socket.timeout.ms=10&bootstrap.servers=broker:29092&group.id=operator.consumer.test.integration.test_1&enable.auto.commit=False&auto.offset.reset=beginning",
        ),
        Connection(
            conn_id="operator.consumer.test.integration.test_2",
            uri="kafka://broker:29092?socket.timeout.ms=10&bootstrap.servers=broker:29092&group.id=operator.consumer.test.integration.test_2&enable.auto.commit=False&auto.offset.reset=beginning",
        ),
        Connection(
            conn_id="operator.consumer.test.integration.test_3",
            uri="kafka://broker:29092?socket.timeout.ms=10&bootstrap.servers=broker:29092&group.id=operator.consumer.test.integration.test_3&enable.auto.commit=False&auto.offset.reset=beginning",
        ),
    ]

    for conn in connections:
        create_connection_without_db(conn)


@pytest.mark.integration("kafka")
class TestConsumeFromTopic:
    """
    test ConsumeFromTopicOperator
    """

    def test_consumer_operator_test_1(self):
        """test consumer works with string import"""

        TOPIC = "operator.consumer.test.integration.test_1"

        p = Producer(**{"bootstrap.servers": "broker:29092"})
        p.produce(TOPIC, TOPIC)
        assert len(p) == 1
        x = p.flush()
        assert x == 0

        operator = ConsumeFromTopicOperator(
            kafka_config_id=TOPIC,
            topics=[TOPIC],
            apply_function="integration.apache.kafka.operators.test_consume._basic_message_tester",
            apply_function_kwargs={"test": TOPIC},
            task_id="test",
            poll_timeout=10,
        )

        x = operator.execute(context={})

    def test_consumer_operator_test_2(self):
        """test consumer works with direct binding"""

        TOPIC = "operator.consumer.test.integration.test_2"

        p = Producer(**{"bootstrap.servers": "broker:29092"})
        p.produce(TOPIC, TOPIC)
        assert len(p) == 1
        x = p.flush()
        assert x == 0

        operator = ConsumeFromTopicOperator(
            kafka_config_id=TOPIC,
            topics=[TOPIC],
            apply_function=_basic_message_tester,
            apply_function_kwargs={"test": TOPIC},
            task_id="test",
            poll_timeout=10,
        )

        x = operator.execute(context={})

    def test_consumer_operator_test_3(self):
        """test consumer works in batch mode"""

        TOPIC = "operator.consumer.test.integration.test_3"

        p = Producer(**{"bootstrap.servers": "broker:29092"})

        for _ in range(20):
            p.produce(TOPIC, TOPIC)

        assert len(p) == 20
        x = p.flush()
        assert x == 0

        operator = ConsumeFromTopicOperator(
            kafka_config_id=TOPIC,
            topics=[TOPIC],
            apply_function_batch=_batch_tester,
            apply_function_kwargs={"test_string": TOPIC},
            task_id="test",
            poll_timeout=10,
            commit_cadence="end_of_batch",
            max_messages=30,
            max_batch_size=10,
        )

        x = operator.execute(context={})



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json

import pytest
from confluent_kafka import Producer

from airflow.models import Connection

# Import Hook
from airflow.providers.apache.kafka.hooks.client import KafkaAdminClientHook
from airflow.providers.apache.kafka.hooks.consume import KafkaConsumerHook

TOPIC = "consumer_hook_test_1"

config = {
    "bootstrap.servers": "broker:29092",
    "group.id": "hook.consumer.integration.test",
    "enable.auto.commit": False,
    "auto.offset.reset": "beginning",
}


@pytest.mark.integration("kafka")
class TestConsumerHook:
    """
    Test consumer hook.
    """

    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(config),
            )
        )

    def test_consume_messages(self):
        """test initialization of AdminClientHook"""

        # Standard Init
        p = Producer(**{"bootstrap.servers": "broker:29092"})
        p.produce(TOPIC, "test_message")
        assert len(p) == 1
        x = p.flush()
        assert x == 0

        c = KafkaConsumerHook([TOPIC], kafka_config_id="kafka_d")
        consumer = c.get_consumer()

        msg = consumer.consume()

        assert msg[0].value() == b"test_message"
        hook = KafkaAdminClientHook(kafka_config_id="kafka_d")
        hook.delete_topic(topics=[TOPIC])



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import asyncio
import json

import pytest

from airflow.models import Connection
from airflow.providers.apache.kafka.hooks.consume import KafkaConsumerHook

from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS, get_base_airflow_version_tuple

if AIRFLOW_V_3_0_PLUS:
    from airflow.providers.apache.kafka.triggers.msg_queue import KafkaMessageQueueTrigger
    from airflow.providers.common.messaging.triggers.msg_queue import MessageQueueTrigger


def apply_function_false(message):
    return False


def apply_function_true(message):
    return True


class MockedMessage:
    def __init__(*args, **kwargs):
        pass

    def error(*args, **kwargs):
        return False


class MockedConsumer:
    def __init__(*args, **kwargs) -> None:
        pass

    def poll(*args, **kwargs):
        return MockedMessage()

    def commit(*args, **kwargs):
        return True


class TestMessageQueueTrigger:
    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(
                    {"socket.timeout.ms": 10, "bootstrap.servers": "localhost:9092", "group.id": "test_group"}
                ),
            )
        )
        create_connection_without_db(
            Connection(
                conn_id="kafka_multi_bootstraps",
                conn_type="kafka",
                extra=json.dumps(
                    {
                        "socket.timeout.ms": 10,
                        "bootstrap.servers": "localhost:9091,localhost:9092,localhost:9093",
                        "group.id": "test_groups",
                    }
                ),
            )
        )

    @pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="Requires Airflow 3.0.+")
    def test_trigger_serialization(self):
        if get_base_airflow_version_tuple() < (3, 0, 1):
            pytest.skip("This test is only for Airflow 3.0.1+")
        trigger = KafkaMessageQueueTrigger(
            kafka_config_id="kafka_d",
            apply_function="test.noop",
            topics=["noop"],
            apply_function_args=[1, 2],
            apply_function_kwargs={"one": 1, "two": 2},
            poll_timeout=10,
            poll_interval=5,
        )

        assert isinstance(trigger, KafkaMessageQueueTrigger)
        assert isinstance(trigger, MessageQueueTrigger)

        classpath, kwargs = trigger.serialize()

        assert classpath == "airflow.providers.apache.kafka.triggers.await_message.AwaitMessageTrigger"
        assert kwargs == {
            "kafka_config_id": "kafka_d",
            "apply_function": "test.noop",
            "topics": ["noop"],
            "apply_function_args": [1, 2],
            "apply_function_kwargs": {"one": 1, "two": 2},
            "poll_timeout": 10,
            "poll_interval": 5,
            "commit_offset": True,
        }

    @pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="Requires Airflow 3.0.+")
    @pytest.mark.asyncio
    async def test_trigger_run_good(self, mocker):
        if get_base_airflow_version_tuple() < (3, 0, 1):
            pytest.skip("This test is only for Airflow 3.0.1+")
        mocker.patch.object(KafkaConsumerHook, "get_consumer", return_value=MockedConsumer)

        trigger = KafkaMessageQueueTrigger(
            kafka_config_id="kafka_d",
            apply_function="unit.apache.kafka.triggers.test_msg_queue.apply_function_true",
            topics=["noop"],
            poll_timeout=0.0001,
            poll_interval=5,
        )

        task = asyncio.create_task(trigger.run().__anext__())
        await asyncio.sleep(1.0)
        assert task.done() is True

    @pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="Requires Airflow 3.0.+")
    @pytest.mark.asyncio
    async def test_trigger_run_bad(self, mocker):
        mocker.patch.object(KafkaConsumerHook, "get_consumer", return_value=MockedConsumer)

        trigger = KafkaMessageQueueTrigger(
            kafka_config_id="kafka_d",
            apply_function="unit.apache.kafka.triggers.test_msg_queue.apply_function_false",
            topics=["noop"],
            poll_timeout=0.0001,
            poll_interval=5,
        )

        task = asyncio.create_task(trigger.run().__anext__())
        await asyncio.sleep(1.0)
        assert task.done() is False



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock
from unittest.mock import MagicMock

import pytest

from airflow.providers.apache.kafka.hooks.base import KafkaBaseHook

try:
    import importlib.util

    if not importlib.util.find_spec("airflow.sdk.bases.hook"):
        raise ImportError

    BASEHOOK_PATCH_PATH = "airflow.sdk.bases.hook.BaseHook"
except ImportError:
    BASEHOOK_PATCH_PATH = "airflow.hooks.base.BaseHook"


class SomeKafkaHook(KafkaBaseHook):
    def _get_client(self, config):
        return config


@pytest.fixture
def hook():
    return SomeKafkaHook()


TIMEOUT = 10


class TestKafkaBaseHook:
    @mock.patch(f"{BASEHOOK_PATCH_PATH}.get_connection")
    def test_get_conn(self, mock_get_connection, hook):
        config = {"bootstrap.servers": MagicMock()}
        mock_get_connection.return_value.extra_dejson = config
        assert hook.get_conn == config

    @mock.patch(f"{BASEHOOK_PATCH_PATH}.get_connection")
    def test_get_conn_value_error(self, mock_get_connection, hook):
        mock_get_connection.return_value.extra_dejson = {}
        with pytest.raises(ValueError, match="must be provided"):
            hook.get_conn()

    @mock.patch("airflow.providers.apache.kafka.hooks.base.AdminClient")
    @mock.patch(f"{BASEHOOK_PATCH_PATH}.get_connection")
    def test_test_connection(self, mock_get_connection, admin_client, hook):
        config = {"bootstrap.servers": MagicMock()}
        mock_get_connection.return_value.extra_dejson = config
        connection = hook.test_connection()
        admin_client.assert_called_once_with(config)
        mock_admin_instance = admin_client.return_value
        mock_admin_instance.list_topics.assert_called_once_with(timeout=TIMEOUT)
        assert connection == (True, "Connection successful.")

    @mock.patch(
        "airflow.providers.apache.kafka.hooks.base.AdminClient",
        return_value=MagicMock(list_topics=MagicMock(return_value=[])),
    )
    @mock.patch(f"{BASEHOOK_PATCH_PATH}.get_connection")
    def test_test_connection_no_topics(self, mock_get_connection, admin_client, hook):
        config = {"bootstrap.servers": MagicMock()}
        mock_get_connection.return_value.extra_dejson = config
        connection = hook.test_connection()
        admin_client.assert_called_once_with(config)
        mock_admin_instance = admin_client.return_value
        mock_admin_instance.list_topics.assert_called_once_with(timeout=TIMEOUT)
        assert connection == (False, "Failed to establish connection.")

    @mock.patch("airflow.providers.apache.kafka.hooks.base.AdminClient")
    @mock.patch(f"{BASEHOOK_PATCH_PATH}.get_connection")
    def test_test_connection_exception(self, mock_get_connection, admin_client, hook):
        config = {"bootstrap.servers": MagicMock()}
        mock_get_connection.return_value.extra_dejson = config
        admin_client.return_value.list_topics.side_effect = [ValueError("some error")]
        connection = hook.test_connection()
        assert connection == (False, "some error")



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
import logging
from unittest.mock import MagicMock, patch

import pytest
from confluent_kafka.admin import AdminClient

from airflow.models import Connection
from airflow.providers.apache.kafka.hooks.produce import KafkaProducerHook

log = logging.getLogger(__name__)


class TestProducerHook:
    """
    Test consumer hook.
    """

    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id="kafka_d",
                conn_type="kafka",
                extra=json.dumps(
                    {"socket.timeout.ms": 10, "bootstrap.servers": "localhost:9092", "group.id": "test_group"}
                ),
            )
        )

        create_connection_without_db(
            Connection(
                conn_id="kafka_bad",
                conn_type="kafka",
                extra=json.dumps({}),
            )
        )
        self.hook = KafkaProducerHook(kafka_config_id="kafka_d")

    @patch("airflow.providers.apache.kafka.hooks.base.AdminClient")
    def test_get_producer(self, mock_client):
        mock_client_spec = MagicMock(spec=AdminClient)
        mock_client.return_value = mock_client_spec
        assert self.hook.get_producer() == self.hook.get_conn



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import pytest

from airflow.providers.apache.kafka.triggers.await_message import AwaitMessageTrigger

pytest.importorskip("airflow.providers.common.messaging.providers.base_provider")

MOCK_KAFKA_TRIGGER_APPLY_FUNCTION = "mock_kafka_trigger_apply_function"


class TestKafkaMessageQueueProvider:
    """Tests for KafkaMessageQueueProvider."""

    def setup_method(self):
        """Set up the test environment."""
        from airflow.providers.apache.kafka.queues.kafka import KafkaMessageQueueProvider

        self.provider = KafkaMessageQueueProvider()

    def test_queue_create(self):
        """Test the creation of the KafkaMessageQueueProvider."""
        from airflow.providers.common.messaging.providers.base_provider import BaseMessageQueueProvider

        assert isinstance(self.provider, BaseMessageQueueProvider)

    @pytest.mark.parametrize(
        ("queue_uri", "expected_result"),
        [
            pytest.param("kafka://localhost:9092/topic1", True, id="single_broker_single_topic"),
            pytest.param(
                "kafka://broker1:9092,broker2:9092/topic1,topic2", True, id="multiple_brokers_multiple_topics"
            ),
            pytest.param("http://example.com", False, id="http_url"),
            pytest.param("not-a-url", False, id="invalid_url"),
        ],
    )
    def test_queue_matches(self, queue_uri, expected_result):
        """Test the queue_matches method with various URLs."""
        assert self.provider.queue_matches(queue_uri) == expected_result

    @pytest.mark.parametrize(
        ("scheme", "expected_result"),
        [
            pytest.param("kafka", True, id="kafka_scheme"),
            pytest.param("redis+pubsub", False, id="redis_scheme"),
            pytest.param("sqs", False, id="sqs_scheme"),
            pytest.param("unknown", False, id="unknown_scheme"),
        ],
    )
    def test_scheme_matches(self, scheme, expected_result):
        """Test the scheme_matches method with various schemes."""
        assert self.provider.scheme_matches(scheme) == expected_result

    def test_trigger_class(self):
        """Test the trigger_class method."""
        assert self.provider.trigger_class() == AwaitMessageTrigger

    @pytest.mark.parametrize(
        ("queue_uri", "extra_kwargs", "expected_result"),
        [
            pytest.param(
                "kafka://broker:9092/topic1,topic2",
                {"apply_function": MOCK_KAFKA_TRIGGER_APPLY_FUNCTION},
                {"topics": ["topic1", "topic2"]},
                id="topics_from_uri",
            ),
            pytest.param(
                "kafka://broker:9092/",
                {"topics": ["topic1", "topic2"], "apply_function": MOCK_KAFKA_TRIGGER_APPLY_FUNCTION},
                {},
                id="topics_from_kwargs",
            ),
        ],
    )
    def test_trigger_kwargs_valid_cases(self, queue_uri, extra_kwargs, expected_result):
        """Test the trigger_kwargs method with valid parameters."""
        kwargs = self.provider.trigger_kwargs(queue_uri, **extra_kwargs)
        assert kwargs == expected_result

    @pytest.mark.parametrize(
        ("queue_uri", "extra_kwargs", "expected_error", "error_match"),
        [
            pytest.param(
                "kafka://broker:9092/topic1",
                {},
                ValueError,
                "apply_function is required in KafkaMessageQueueProvider kwargs",
                id="missing_apply_function",
            ),
            pytest.param(
                "kafka://broker:9092/",
                {"apply_function": MOCK_KAFKA_TRIGGER_APPLY_FUNCTION},
                ValueError,
                "topics is required in KafkaMessageQueueProvider kwargs or provide them in the queue URI",
                id="missing_topics",
            ),
        ],
    )
    def test_trigger_kwargs_error_cases(self, queue_uri, extra_kwargs, expected_error, error_match):
        """Test that trigger_kwargs raises appropriate errors with invalid parameters."""
        with pytest.raises(expected_error, match=error_match):
            self.provider.trigger_kwargs(queue_uri, **extra_kwargs)
